#  Copyright (c) 2019-2021, Andrey "Limych" Khrolenok <andrey@khrolenok.ru>
#  Creative Commons BY-NC-SA 4.0 International Public License
#  (see LICENSE.md or https://creativecommons.org/licenses/by-nc-sa/4.0/)
"""Constants."""

# Base library constants
NAME = "Beward Client"
VERSION = "1.1.2"
AUTHOR = 'Andrey "Limych" Khrolenok <andrey@khrolenok.ru>'
LICENSE = "Creative Commons BY-NC-SA License"
WEBSITE = "https://github.com/Limych/py-beward"
ISSUE_URL = "https://github.com/Limych/py-beward/issues"

STARTUP_MESSAGE = f"""
-------------------------------------------------------------------
{NAME}
Version: {VERSION}
If you have ANY issues with this you need to open an issue here:
{ISSUE_URL}
-------------------------------------------------------------------
"""


TIMEOUT = 3

# Error strings
MSG_GENERIC_FAIL = "Sorry.. Something went wrong..."

# Alarms
ALARM_ONLINE = "DeviceOnline"
ALARM_MOTION = "MotionDetection"
ALARM_SENSOR = "SensorAlarm"
ALARM_SENSOR_OUT = "SensorOutAlarm"

# Device types
BEWARD_CAMERA = "camera"
BEWARD_DOORBELL = "doorbell"

# Beward models
BEWARD_MODELS = {
    BEWARD_CAMERA: "",
    BEWARD_DOORBELL: "DK103 DK103M DS03M DS05M(P) DS06M DS06A(P) DSN06PS S06A S06M ",
}



#  Copyright (c) 2019-2021, Andrey "Limych" Khrolenok <andrey@khrolenok.ru>
#  Creative Commons BY-NC-SA 4.0 International Public License
#  (see LICENSE.md or https://creativecommons.org/licenses/by-nc-sa/4.0/)
"""Python API for Beward Cameras and Doorbells."""

import logging
import struct
from collections import namedtuple

import hexdump
from _socket import (
    AF_INET,
    SO_BROADCAST,
    SO_REUSEADDR,
    SOCK_DGRAM,
    SOL_SOCKET,
    inet_ntoa,
    socket,
    timeout,
)

# Will be parsed by setup.py to determine package metadata
from beward.camera import BewardCamera
from beward.const import BEWARD_CAMERA, BEWARD_DOORBELL, STARTUP_MESSAGE
from beward.core import BewardGeneric
from beward.doorbell import BewardDoorbell

# You really should not `import *` - it is poor practice
# but if you do, here is what you get:
__all__ = [
    "Beward",
    "BewardGeneric",
    "BewardCamera",
    "BewardDoorbell",
]

_LOGGER = logging.getLogger(__name__)
#
# http://docs.python.org/2/howto/logging.html#library-config
# Avoids spurious error messages if no logger is configured by the user
_LOGGER.addHandler(logging.NullHandler())
#
_LOGGER.info(STARTUP_MESSAGE)


# pylint: disable=too-few-public-methods
class Beward:
    """Beward device factory class."""

    @staticmethod
    def discovery():  # pragma: no cover
        """Discover Beward devices in local network."""
        server = socket(AF_INET, SOCK_DGRAM)
        server.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
        server.setsockopt(SOL_SOCKET, SO_BROADCAST, 1)
        server.bind(("0.0.0.0", 0))
        server.settimeout(1)

        _LOGGER.debug("Start discovery")
        server.sendto(
            b"\x67\x45\x00\x00\x05\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
            b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",
            ("255.255.255.255", 59123),
        )

        devices = {}
        while True:
            try:
                data = server.recvfrom(1024)
                _LOGGER.debug(
                    "Discovery response data:\n%s",
                    hexdump.hexdump(data[0][28:], result="return"),
                )

                (
                    # packet header (28 bytes):
                    # "\x67\x45\x00\x00\x05\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x48\x02\x00\x00" (packet data length) = 584
                    # packet data (584 bytes):
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x00"
                    device_id,  # "\x5f\x06\x00\x00"
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x03"
                    # "\x21\x00\x00"
                    name,  # "\x49\x50\x43\x31\x37\x32\x34\x00\x00\x00\x00..."
                    host_ip,  # "\x5a\x01\xa8\xc0"
                    mac,  # "\x00\x5a\x22\x30\x07\x5f"
                    http_port,  # "\x50\x00"
                    data_port,  # "\x88\x13"
                    # "\x00\x00"
                    net_mask,  # "\x00\xff\xff\xff"
                    gate_ip,  # "\x01\x01\xa8\xc0"
                    # "\x01\x08\x37\xe0"
                    # "\x01\x01\xa8\xc0" (gate_ip)
                    # "\x88\x13" (data_port)
                    # "\x00\x00\x01\x00\x00\x00"
                    # "\x5a\x01\xa8\xc0" (host_ip)
                    # "\x00\xff\xff\xff" "\x01\x01\xa8\xc0" (net_mask + gate_ip)
                    # "\x88\x13" "\x50\x00" (data_port + http_port)
                    # "\x01\x08\x37\xe0"
                    # "\x88\x13" (data_port)
                    # "\x00\x5a\x22\x30\x07\x5f" (mac)
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x02\x30\x75"
                    # "\x50\x00" "\x88\x13" (http_port + data_port)
                    # "\x00\x00"
                    # "\x01\x01\xa8\xc0" (gate_ip = dns1_ip?)
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x00\x00\x00\x70\x17\x37\x01\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x00"
                    # "\x08\x08\x08\x08" (dns2_ip)
                    # "\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x80\x00"
                    # "\x00\x00\x00\x00\x01\x00"
                    # "\xa0\x01\xa8\xc0" (ip?)
                    # "\x00\xff\xff\xff" "\x01\x01\xa8\xc0" (net_mask + gate_ip)
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    # "\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x38\x71\x32\x4d"
                    # "\x75\x49\x62\x6d\x7a\x32\x67\x66\x4c\x5a\x35\x70\x6d\x42\x54\x51"
                    # "\x49\x69\x49\x77\x6f\x37\x63\x71\x6c\x4e\x64\x30"
                ) = struct.unpack("<45xL19x64sI6s2H2x2I", data[0][:156])
                name = name.replace(b"\x00", b"").decode("utf-8")

                def _unpack_ip(ip_addr):
                    return inet_ntoa(struct.pack(">I", ip_addr))

                (host_ip, net_mask, gate_ip) = (
                    _unpack_ip(host_ip),
                    _unpack_ip(net_mask),
                    _unpack_ip(gate_ip),
                )
                # pylint: disable=consider-using-f-string
                mac = ":".join("%02x" % i for i in mac)

                _LOGGER.info(
                    "Discovered %s (ID: %d) at http://%s:%d",
                    name,
                    device_id,
                    host_ip,
                    http_port,
                )

                if mac not in devices:
                    dev = {
                        "device_id": device_id,
                        "name": name,
                        "host_ip": host_ip,
                        "http_port": http_port,
                        "data_port": data_port,
                        "mac": mac,
                        "net_mask": net_mask,
                        "gate_ip": gate_ip,
                    }
                    devices[mac] = namedtuple("BewardDevice", dev.keys())(*dev.values())

            except Exception as err:  # pylint: disable=broad-except
                if not isinstance(err, timeout):
                    _LOGGER.debug(err)
                break

        _LOGGER.debug("Stop discovery")
        server.close()

        return devices

    @staticmethod
    def factory(host_ip: str, username: str, password: str, **kwargs):
        """Return correct class for device."""
        bwd = BewardGeneric(host_ip, username, password)
        model = bwd.system_info.get("DeviceModel")
        dev_type = bwd.get_device_type(model)

        if dev_type is None:
            raise ValueError(f'Unknown device "{model}"')

        inst = None

        if dev_type == BEWARD_CAMERA:
            inst = BewardCamera(host_ip, username, password, **kwargs)

        elif dev_type == BEWARD_DOORBELL:
            inst = BewardDoorbell(host_ip, username, password, **kwargs)

        _LOGGER.debug("Factory create instance of %s", inst.__class__)
        return inst



#!/usr/bin/env python
"""Library module setup."""

import re
import sys

from setuptools import find_packages, setup
from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):
    """PyTest controller."""

    # Code from here:
    # https://docs.pytest.org/en/latest/goodpractices.html#manual-integration

    # pylint: disable=attribute-defined-outside-init
    def finalize_options(self):
        """Finalize test command options."""
        TestCommand.finalize_options(self)
        # we don't run integration tests which need an actual beward device
        self.test_args = ["-m", "not integration"]
        self.test_suite = True

    # pylint: disable=import-outside-toplevel,import-error
    def run_tests(self):
        """Run tests."""
        # import here, cause outside the eggs aren't loaded
        import shlex

        import pytest

        errno = pytest.main(shlex.split(self.pytest_args))
        sys.exit(errno)


def load_requirements(fpath: str) -> list:
    """Load requirements from file."""
    data = list(open(fpath))
    imp = re.compile(r"^(-r|--requirement)\s+(\S+)")
    reqs = []
    for i in data:
        # pylint: disable=invalid-name
        m = imp.match(i)
        if m:
            reqs.extend(load_requirements(m.group(2)))
        else:
            reqs.append(i)

    return reqs


src = open("beward/const.py", encoding="utf-8").read()
metadata = dict(re.findall(r'([a-z]+) = "([^"]+)"', src, re.IGNORECASE))
metadata.update(dict(re.findall(r"([a-z]+) = '([^']+)'", src, re.IGNORECASE)))
docstrings = re.findall(r'"""(.*?)"""', src, re.MULTILINE | re.DOTALL)

NAME = "beward"

PACKAGES = [x for x in find_packages() if x not in ["bin", "tests"]]

VERSION = metadata["VERSION"]
AUTHOR_EMAIL = metadata.get("AUTHOR", "Unknown <no@email.com>")
WEBSITE = metadata.get("WEBSITE", "")
LICENSE = metadata.get("LICENSE", "")
DESCRIPTION = docstrings[0]

CLASSIFIERS = [
    "Development Status :: 5 - Production/Stable",
    "Intended Audience :: Developers",
    "License :: Other/Proprietary License",
    "Operating System :: OS Independent",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3.10",
    "Programming Language :: Python :: Implementation :: CPython",
    "Topic :: Home Automation",
    "Topic :: Security",
    "Topic :: Multimedia :: Video :: Capture",
]

with open("README.md", encoding="utf-8") as file:
    LONG_DESCRIPTION = file.read()
    LONG_DESCRIPTION_TYPE = "text/markdown"

# Extract name and e-mail ("Firstname Lastname <mail@example.org>")
AUTHOR, EMAIL = re.match(r"(.*) <(.*)>", AUTHOR_EMAIL).groups()

REQUIREMENTS = load_requirements("requirements.txt")
TEST_REQUIREMENTS = load_requirements("requirements-test.txt")

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    author=AUTHOR,
    author_email=EMAIL,
    license=LICENSE,
    url=WEBSITE,
    packages=PACKAGES,
    install_requires=REQUIREMENTS,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESCRIPTION_TYPE,
    classifiers=CLASSIFIERS,
    cmdclass={"pytest": PyTest},
    test_suite="tests",
    tests_require=TEST_REQUIREMENTS,
)
